/* Controller.m */

#import "Controller.h"

NSString *ltob(unsigned long val)
{
    int i;
    char buf[33];

    for (i=0; i<32; i++) {
        buf[i] = (val & (1<<(31-i)) ? '1' : '0');
    }
    buf[32] = '\0';

    for (i=0; i<32; i++) {
        if (buf[i] != '0'){
           return [NSString stringWithCString:buf+i];
        }
    }
    return [NSString stringWithCString:buf+31];
}


@implementation Controller

- (IBAction)clear:(id)sender 
{
   X = 0.0;
   [self displayX];
}


- (IBAction)clearAll:(id)sender 
{
   X = 0.0;
   Y = 0.0;
   yFlag = NO;
   enterFlag = NO;
   [self displayX];
}


- (IBAction)enterDigit:(id)sender 
{
   if (enterFlag) {
     Y = X;
     X = 0.0;
     enterFlag = NO;
   }

   X = (X*radix) + [ [sender selectedCell] tag];
   [self displayX];
}



- (IBAction)enterOp:(id)sender 
{
  if (yFlag) {           // Something is stored in Y
    switch (operation) {
       case PLUS:
         X = Y + X;
         break;

       case SUBTRACT:
         X = Y - X;
         break;

       case MULTIPLY:
         X = Y * X;
         break;

       case DIVIDE:
         X = Y / X;
         break;
     }
  }

   Y    = X;
   yFlag = YES;

  operation = [ [sender selectedCell] tag];
  enterFlag = YES;

  [self displayX];
}


//- (void)displayX       FOR CHAPS 5-6 ONLY
//{
//   id s = [NSString stringWithFormat:@"%15.10g", X ];
//   [readout setStringValue: s];
//}

- (void)displayX     // New as of Chap. 7
{
    NSString *s=nil;

    switch(radix){
    case 16:
        s = [NSString stringWithFormat:@"%x",(int)X];
        break;
    case 10:
        s = [NSString stringWithFormat:@"%15.10g",X];
        break;
    case 8:
        s = [NSString stringWithFormat:@"%o",(int)X];
        break;
    case 2:
        s = ltob((int)X);
        break;
    }

    [readout setStringValue: s];
}




- (IBAction)doUnaryMinus:(id)sender
{
   X = -X;
   [self displayX];
}


- (IBAction)showAboutPanel:(id)sender
{
    if (aboutPanel == nil) {
        if ( ![NSBundle loadNibNamed:@"AboutPanel.nib" owner:self] ) {
           NSLog(@"Load of AboutPanel.nib failed");
           return;
        }
     }
     
    [aboutPanel makeKeyAndOrderFront: nil];
}



//- (IBAction)setRadix:(id)sender     COMMENTED OUT IN MIDDLE OF CHAP 7
//{
//    radix = [[sender selectedCell] tag];
//    [self displayX];
//}

/*
- (IBAction)setRadix:(id)sender        COMMENTED OUT IN MIDDLE OF CHAP 7
{
    NSArray *cells;
    int i;

    radix = [[sender selectedCell] tag];

    // Disable the buttons that are higher than selected radix
    cells = [keyPad cells];
    for (i=0; i<[cells count]; i++) {
        id cell = [cells objectAtIndex: i];
        [cell setEnabled: ([cell tag] < radix) ];
    }

    [self displayX];
}
*/




- (IBAction)setRadix:(id)sender
{
    NSEnumerator *enumerator;
    NSCell *cell;
    int oldRadix = radix;

    radix = [ [sender selectedCell] tag];

    if (radix!=oldRadix && (radix==16 || oldRadix==16) ) {

        double ysize = [keyPad cellSize].height * 2
                         + [keyPad intercellSpacing].height * 2;
        int row,col;
        NSWindow *win = [keyPad window];
        NSRect frame = [win frame];

        // If switching to radix 16, grow the window,
        // and keep the title bar in the same place.
        if (radix==16){
            frame.size.height += ysize;
            frame.origin.y    -= ysize;
            [win setFrame:frame display:YES animate:YES];

            for (row=0;row<2;row++) {
                [keyPad insertRow:0];

                for(col=0;col<3;col++){
                    int val = 10 + row*3 + col;
                    cell = [keyPad cellAtRow:0 column:col];
                    [cell setTag:val];
                    [cell setTitle:[NSString
                                    stringWithFormat:@"%X",val]];
                }
            }

            [keyPad sizeToCells];
            [keyPad setNeedsDisplay];
        }

        // If switching away from base 16, shrink the window
        // (keeping the title bar in the same place)
        else {
            frame.size.height -= ysize;
            frame.origin.y    += ysize;
            [keyPad removeRow:0];
            [keyPad removeRow:0];
            [keyPad sizeToCells];
            [keyPad setNeedsDisplay];
            [win setFrame:frame display:YES animate:YES];
        }
    }

    // Disable the buttons that are higher than selected radix
    enumerator = [ [keyPad cells] objectEnumerator];

    while (cell = [enumerator nextObject]) {
        [cell setEnabled: ([cell tag] < radix) ];
    }
    [self displayX];
}



@end



@implementation Controller(ApplicationNotifications)

-(void)applicationDidFinishLaunching: (NSNotification*)notification
{
    radix = [ [radixPopUp selectedItem] tag];
    [self clearAll:self];
}
@end



