#import <Cocoa/Cocoa.h>

@interface GraphView : NSView
{
    IBOutlet id formulaField;
    IBOutlet id graphButton;
    IBOutlet id xmaxCell;
    IBOutlet id xminCell;
    IBOutlet id xstepCell;
    IBOutlet id ymaxCell;
    IBOutlet id yminCell;

    // These five variables are the same as those in MathPaper
    NSPipe       *toPipe;
    NSPipe       *fromPipe;
    NSFileHandle *toEvaluator;
    NSFileHandle *fromEvaluator;
    NSTask       *evaluator;

    NSMutableString     *fromBuf;

    // These hold the contents of the NSForm
    double      xmin;
    double      xmax;
    double      xstep;
    double      ymin;
    double      ymax;

    // Display list and glue
    NSMutableArray *displayList;
    BOOL            first;        // Getting the first point?
    NSPoint         lastPt;       // Last point received

    // Communication with stuffer thread
    BOOL       stop_sending;
    BOOL       sending;
    BOOL       receiving;

    BOOL      graphing;
}

- (IBAction)graph:(id)sender;
- (IBAction)stopGraph:(id)sender;
- (void)doStop:(int)which;
- (void)getFormAndScaleView;
- (void)addGraphElement:(id)element;
- (void)clear;
- (void)sendData;
- (void)addAxesFrom:(NSPoint)pt1 to:(NSPoint)pt2;


@end


@protocol GraphViewElement
- (int)tag;
- (void)setTag:(int)aTag;
- (void)stroke;
- (NSRect)bounds;
- (void)setColor:(NSColor *)aColor;
- (NSColor *)color;
@end


#define STOP_SENDER 1
#define STOP_RECEIVER 2

#define GRAPH_TAG 1
#define AXES_TAG  2
#define LABEL_TAG 3
